@@ -448,7 +448,7 @@ spi_dec_button_update_and_emit (SpiDEController *controller,
 	spi_init_any_nil (&e.any_data, 
 		    spi_accessible_new_return (atk_get_root (), FALSE, NULL),
 		    Accessibility_ROLE_UNKNOWN,
-		    CORBA_string_dup (""));
+		    "");
 	CORBA_exception_init (&ev);
 	if (!is_consumed)
 	  {
@@ -458,6 +458,8 @@ spi_dec_button_update_and_emit (SpiDEController *controller,
 	  }
 	else
 	  spi_dec_set_unlatch_pending (controller, mask_return);
+
+        CORBA_free (e.any_data._value);
       }
       return TRUE;
     }
@@ -473,6 +475,7 @@ spi_dec_mouse_check (SpiDEController *controller,
 		     int *x, int *y, gboolean *moved)
 {
   Accessibility_Event e;
+  Accessibility_EventDetails *details;
   CORBA_Environment ev;
   int win_x_return,win_y_return;
   unsigned int mask_return;
@@ -505,11 +508,13 @@ spi_dec_mouse_check (SpiDEController *controller,
       spi_init_any_nil (&e.any_data,
 		    spi_accessible_new_return (atk_get_root (), FALSE, NULL),
 		    Accessibility_ROLE_UNKNOWN,
-		    CORBA_string_dup (""));
+		    "");
       CORBA_exception_init (&ev);
       Accessibility_Registry_notifyEvent (BONOBO_OBJREF (controller->registry),
 					  &e,
 					  &ev);
+      details = e.any_data._value;
+      CORBA_free (details);
       e.type = "mouse:rel";  
       e.source = BONOBO_OBJREF (controller->registry->desktop);
       e.detail1 = *x - last_mouse_pos->x;
@@ -517,13 +522,15 @@ spi_dec_mouse_check (SpiDEController *controller,
       spi_init_any_nil (&e.any_data,
 		    spi_accessible_new_return (atk_get_root (), FALSE, NULL),
 		    Accessibility_ROLE_UNKNOWN,
-		    CORBA_string_dup (""));
+		    "");
       CORBA_exception_init (&ev);
       last_mouse_pos->x = *x;
       last_mouse_pos->y = *y;
       Accessibility_Registry_notifyEvent (BONOBO_OBJREF (controller->registry),
 					  &e,
 					  &ev);
+      details = e.any_data._value;
+      CORBA_free (details);
       *moved = True;
     }
   else
@@ -559,11 +566,12 @@ spi_dec_emit_modifier_event (SpiDEController *controller, guint prev_mask,
   spi_init_any_nil (&e.any_data,
 		    spi_accessible_new_return (atk_get_root (), FALSE, NULL),
 		    Accessibility_ROLE_UNKNOWN,
-		    CORBA_string_dup (""));
+		    "");
   CORBA_exception_init (&ev);
   Accessibility_Registry_notifyEvent (BONOBO_OBJREF (controller->registry),
 				      &e,
 				      &ev);
+  CORBA_free (e.any_data._value);
 }
 
 static gboolean
@@ -1077,12 +1085,13 @@ spi_device_event_controller_forward_mouse_event (SpiDEController *controller,
       spi_init_any_nil (&e.any_data,
 		    spi_accessible_new_return (atk_get_root (), FALSE, NULL),
 		    Accessibility_ROLE_UNKNOWN,
-		    CORBA_string_dup (""));
+		    "");
       CORBA_exception_init (&ev);
       
       Accessibility_Registry_notifyEvent (BONOBO_OBJREF (controller->registry),
 					  &e,
 					  &ev);
+      CORBA_free (e.any_data._value);
     }
 
   xkb_mod_unlatch_occurred = (xevent->type == ButtonPress ||
@@ -1426,7 +1435,7 @@ spi_key_event_matches_listener (const Accessibility_DeviceEvent *key_event,
 
 static gboolean
 spi_controller_notify_keylisteners (SpiDEController                 *controller,
-				    const Accessibility_DeviceEvent *key_event,
+				    Accessibility_DeviceEvent       *key_event,
 				    CORBA_boolean                    is_system_global,
 				    CORBA_Environment               *ev)
 {
@@ -1440,6 +1449,10 @@ spi_controller_notify_keylisteners (SpiDEController                 *controller,
       return FALSE;
     }
 
+  /* set the NUMLOCK event mask bit if appropriate: see bug #143702 */
+  if (key_event->modifiers & _numlock_physical_mask)
+      key_event->modifiers |= SPI_KEYMASK_NUMLOCK;
+
   for (l = *key_listeners; l; l = l->next)
     {
        DEControllerKeyListener *key_listener = l->data;
@@ -2377,7 +2390,9 @@ impl_notify_listeners_sync (PortableServer_Servant           servant,
   g_print ("notifylistening listeners synchronously: controller %p, event id %d\n",
 	   controller, (int) event->id);
 #endif
-  return spi_controller_notify_keylisteners (controller, event, CORBA_FALSE, ev) ?
+  return spi_controller_notify_keylisteners (controller,
+					     (Accessibility_DeviceEvent *) 
+					     event, CORBA_FALSE, ev) ?
 	  CORBA_TRUE : CORBA_FALSE; 
 }
 
@@ -2392,7 +2407,8 @@ impl_notify_listeners_async (PortableServer_Servant           servant,
 #ifdef SPI_DEBUG
   fprintf (stderr, "notifying listeners asynchronously\n");
 #endif
-  spi_controller_notify_keylisteners (controller, event, CORBA_FALSE, ev); 
+  spi_controller_notify_keylisteners (controller, (Accessibility_DeviceEvent *)
+				      event, CORBA_FALSE, ev); 
 }
 
 static void
@@ -2502,6 +2518,7 @@ spi_device_event_controller_forward_key_event (SpiDEController *controller,
 {
   CORBA_Environment ev;
   Accessibility_DeviceEvent key_event;
+  gboolean ret;
 
   g_assert (event->type == KeyPress || event->type == KeyRelease);
 
@@ -2513,7 +2530,9 @@ spi_device_event_controller_forward_key_event (SpiDEController *controller,
     spi_controller_update_key_grabs (controller, &key_event);
 
   /* relay to listeners, and decide whether to consume it or not */
-  return spi_controller_notify_keylisteners (controller, &key_event, CORBA_TRUE, &ev);
+  ret = spi_controller_notify_keylisteners (controller, &key_event, CORBA_TRUE, &ev);
+  CORBA_free(key_event.event_string);
+  return ret;
 }
 
 SpiDEController *
